/**
 * L2FProd.com Common Components 0.2-dev License.
 *
 * Copyright 2005 L2FProd.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.starbeans.modules.startip;

import java.awt.EventQueue;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.Preferences;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import org.openide.cookies.InstanceCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.util.BaseUtilities;
import org.openide.util.NbPreferences;
import org.starbeans.modules.startip.plaf.LookAndFeelAddons;
import org.starbeans.modules.startip.plaf.aqua.AquaLookAndFeelAddons;
import org.starbeans.modules.startip.plaf.basic.BasicLookAndFeelAddons;
import org.starbeans.modules.startip.plaf.windows.WindowsLookAndFeelAddons;

public class TipManager extends JPanel {
    
    private static final Logger LOGGER = Logger.getLogger(TipManager.class.getName());
    
    private static final int TIMEBASE = 5;
    
    /** Time to show if TIMEBASE x n x n
     * 
     * @param force
     * @return 
     */
    public static boolean timetoShow(boolean force) {
        Preferences tipprefs = NbPreferences.forModule(TipManager.class);
        if(force) {
            tipprefs.putInt("tipoftheday.recurrence", 1);
            return true;
        }
        
        int recu = tipprefs.getInt("tipoftheday.recurrence", 0);
        if(recu==0) {
            tipprefs.putInt("tipoftheday.recurrence", 1);
            return true;
        }
        
        if (recu % 5 != 0) {
            recu++;
            tipprefs.putInt("tipoftheday.recurrence", recu);
            return false;
        }
        
        double n = recu/TIMEBASE;
        recu++;
        tipprefs.putInt("tipoftheday.recurrence", recu);
        n = Math.sqrt(n);
        if(n%1==0) {
            return true;
        }
        return false;
    };
    
    public static boolean getShowingOnStartupPref() {
        Preferences tipprefs = NbPreferences.forModule(TipManager.class);
        boolean b = tipprefs.getBoolean("tipoftheday.showingonstartup", true);
        return b;
    };
    
    public static void setShowingOnStartupPref(boolean showing) {
        Preferences tipprefs = NbPreferences.forModule(TipManager.class);
        tipprefs.putBoolean("tipoftheday.showingonstartup", showing);
    }
    
    public static int getNextTips(int tipcount) {
        Preferences tipprefs = NbPreferences.forModule(TipManager.class);
        int current = tipprefs.getInt("tipoftheday.currenttip", 0);
        if(current<tipcount) {
            tipprefs.putInt("tipoftheday.currenttip", current+1);
        } else {
            tipprefs.putInt("tipoftheday.currenttip", 0);
            current = 0;
        }
        return current;
    }
    
    private static final JTipOfTheDay.ShowOnStartupChoice startupchoice = new JTipOfTheDay.ShowOnStartupChoice() {

            @Override
            public boolean isShowingOnStartup() {
                return getShowingOnStartupPref();
            }

            @Override
            public void setShowingOnStartup(boolean showOnStartup) {
                setShowingOnStartupPref(showOnStartup);
            }
        };

    /**
     *
     * @param force If false, the tips will be shown depending on the 
     * recursivity value.
     * @throws ClassNotFoundException
     */
    public static void showTipoftheDay(boolean force) throws ClassNotFoundException {
        if(!timetoShow(force))
            return;
        
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            if(BaseUtilities.isWindows()) {
                LookAndFeelAddons.setAddon(WindowsLookAndFeelAddons.class.getName());
            } else if(BaseUtilities.isMac()) {
                LookAndFeelAddons.setAddon(AquaLookAndFeelAddons.class.getName());
            } else {
                LookAndFeelAddons.setAddon(BasicLookAndFeelAddons.class.getName());
            }
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex) {
            LOGGER.log(Level.INFO, "",ex);
        }

        DefaultTipModel model = new DefaultTipModel();

        //Load tips
        DataFolder fobj = findTipFolder();
        if(fobj==null)
            return;
             
        DataObject[] darray = fobj.getChildren();
        for (DataObject darray1 : darray) {
            InstanceCookie cookie = darray1.getLookup().lookup(InstanceCookie.class);
            try {
                Object o = cookie.instanceCreate();
                if(o instanceof TipProvider) {
                    TipProvider tp = (TipProvider)o;
                    TipModel tm = tp.getTip();
                    for(int i=0;i<tm.getTipCount();i++) {
                        model.add(tm.getTipAt(i));
                    }
                }
            } catch (IOException ex) {
                LOGGER.log(Level.INFO, "",ex);
                setShowingOnStartupPref(false);
                return;
            }
        }

        int tipcount = model.getTipCount();
        if(tipcount==0) {
            return;
        }
        int currenttip = getNextTips(tipcount);
        
        final JTipOfTheDay totd = new JTipOfTheDay(model);
        totd.setCurrentTip(currenttip);
        
        if (EventQueue.isDispatchThread()) {
            showTipDialog(totd);
        } else {
            EventQueue.invokeLater(new Runnable() {

                @Override
                public void run() {
                    showTipDialog(totd);
                }
            });

        }
    }
    
    private static void showTipDialog(JTipOfTheDay totd) {
        totd.showDialog(new JFrame("title"), startupchoice,true);
    }
    
    /**
     * Returns "Tips" folder.
     */
    private static DataFolder findTipFolder() {
        FileObject fo = FileUtil.getConfigFile("Tips"); //NOI18N
        if (fo == null) {
            IllegalStateException ex = new IllegalStateException();
            LOGGER.log(Level.INFO,"Tip folder not found in layer",ex); //NOI18N
            return null;
        }
        DataFolder df = DataFolder.findFolder(fo);
        return df;
    }
    
    private TipManager() {
    }
}
